require "helpers"

local tables = {}

tables.amenity_point = osm2pgsql.define_table({
    name = 'amenity_point',
    schema = schema_name,
    ids = { type = 'node', id_column = 'osm_id' },
    columns = {
        { column = 'osm_type',     type = 'text', not_null = true },
        { column = 'name',     type = 'text' },
        { column = 'housenumber', type = 'text'},
        { column = 'street',     type = 'text' },
        { column = 'city',     type = 'text' },
        { column = 'state', type = 'text'},
        { column = 'postcode', type = 'text'},
        { column = 'address', type = 'text', not_null = true},
        { column = 'wheelchair', type = 'text'},
        { column = 'wheelchair_desc', type = 'text'},
        { column = 'geom',     type = 'point' , projection = srid},
    }
})

tables.amenity_line = osm2pgsql.define_table({
    name = 'amenity_line',
    schema = schema_name,
    ids = { type = 'way', id_column = 'osm_id' },
    columns = {
        { column = 'osm_type',     type = 'text', not_null = true },
        { column = 'name',     type = 'text' },
        { column = 'housenumber', type = 'text'},
        { column = 'street',     type = 'text' },
        { column = 'city',     type = 'text' },
        { column = 'state', type = 'text'},
        { column = 'postcode', type = 'text'},
        { column = 'address', type = 'text', not_null = true},
        { column = 'wheelchair', type = 'text'},
        { column = 'wheelchair_desc', type = 'text'},
        { column = 'geom',     type = 'linestring' , projection = srid},
    }
})


tables.amenity_polygon = osm2pgsql.define_table({
    name = 'amenity_polygon',
    schema = schema_name,
    ids = { type = 'area', id_column = 'osm_id' },
    columns = {
        { column = 'osm_type',     type = 'text', not_null = true },
        { column = 'name',     type = 'text' },
        { column = 'housenumber', type = 'text'},
        { column = 'street',     type = 'text' },
        { column = 'city',     type = 'text' },
        { column = 'state', type = 'text'},
        { column = 'postcode', type = 'text'},
        { column = 'address', type = 'text', not_null = true},
        { column = 'wheelchair', type = 'text'},
        { column = 'wheelchair_desc', type = 'text'},
        { column = 'geom',     type = 'multipolygon' , projection = srid},
    }
})


-- Keys to include for further checking.  Not all values from each key will be preserved
local amenity_first_level_keys = {
    'amenity',
    'bench',
    'brewery'
}

local is_first_level_amenity = make_check_in_list_func(amenity_first_level_keys)


local function get_osm_type(object)
    local osm_type = object.tags.amenity

    if osm_type == nil and object.tags.bench == 'yes' then
        osm_type = 'bench'
    elseif osm_type == nil and object.tags.brewery then
        osm_type = 'brewery'
    end

    return osm_type
end


function amenity_process_node(object)
    if not is_first_level_amenity(object.tags) then
        return
    end

    local osm_type = get_osm_type(object)

    if osm_type == nil then
        return
    end

    local name = get_name(object.tags)


    local housenumber  = object.tags['addr:housenumber']
    local street = object.tags['addr:street']
    local city = object.tags['addr:city']
    local state = object.tags['addr:state']
    local postcode = object.tags['addr:postcode']
    local address = get_address(object.tags)
    local wheelchair = object.tags.wheelchair
    local wheelchair_desc = get_wheelchair_desc(object.tags)

    tables.amenity_point:add_row({
        osm_type = osm_type,
        name = name,
        housenumber = housenumber,
        street = street,
        city = city,
        state = state,
        postcode = postcode,
        address = address,
        wheelchair = wheelchair,
        wheelchair_desc = wheelchair_desc,
        geom = { create = 'point' }
    })

end

-- Change function name here
function amenity_process_way(object)
    if not is_first_level_amenity(object.tags) then
        return
    end

    local osm_type = get_osm_type(object)

    if osm_type == nil then
        return
    end

    local name = get_name(object.tags)

    local housenumber  = object.tags['addr:housenumber']
    local street = object.tags['addr:street']
    local city = object.tags['addr:city']
    local state = object.tags['addr:state']
    local postcode = object.tags['addr:postcode']
    local address = get_address(object.tags)
    local wheelchair = object.tags.wheelchair
    local wheelchair_desc = get_wheelchair_desc(object.tags)

    if object.is_closed then
        tables.amenity_polygon:add_row({
            osm_type = osm_type,
            name = name,
            housenumber = housenumber,
            street = street,
            city = city,
            state = state,
            postcode = postcode,
            address = address,
            wheelchair = wheelchair,
            wheelchair_desc = wheelchair_desc,
            geom = { create = 'area' }
        })
    else
        tables.amenity_line:add_row({
            osm_type = osm_type,
            name = name,
            housenumber = housenumber,
            street = street,
            city = city,
            state = state,
            postcode = postcode,
            address = address,
            wheelchair = wheelchair,
            wheelchair_desc = wheelchair_desc,
            geom = { create = 'line' }
        })
    end
    
end


function amenity_process_relation(object)
    if not is_first_level_amenity(object.tags) then
        return
    end

    local osm_type = get_osm_type(object)

    if osm_type == nil then
        return
    end

    local name = get_name(object.tags)

    local address = get_address(object.tags)
    local wheelchair = object.tags.wheelchair
    local wheelchair_desc = get_wheelchair_desc(object.tags)

    if object.tags.type == 'multipolygon' or object.tags.type == 'boundary' then
        tables.amenity_polygon:add_row({
            osm_type = osm_type,
            name = name,
            housenumber = housenumber,
            street = street,
            city = city,
            state = state,
            postcode = postcode,
            address = address,
            wheelchair = wheelchair,
            wheelchair_desc = wheelchair_desc,
            geom = { create = 'area' }
        })
    end
end



if osm2pgsql.process_node == nil then
    osm2pgsql.process_node = amenity_process_node
else
    local nested = osm2pgsql.process_node
    osm2pgsql.process_node = function(object)
        local object_copy = deep_copy(object)
        nested(object)
        amenity_process_node(object_copy)
    end
end



if osm2pgsql.process_way == nil then
    osm2pgsql.process_way = amenity_process_way
else
    local nested = osm2pgsql.process_way
    osm2pgsql.process_way = function(object)
        local object_copy = deep_copy(object)
        nested(object)
        amenity_process_way(object_copy)
    end
end


if osm2pgsql.process_relation == nil then
    osm2pgsql.process_relation = amenity_process_relation
else
    local nested = osm2pgsql.process_relation
    osm2pgsql.process_relation = function(object)
        local object_copy = deep_copy(object)
        nested(object)
        amenity_process_relation(object_copy)
    end
end
